\begin{code}
{-# OPTIONS --without-K --safe #-}

module VFP.Bijection.Properties where

open import VFP.Bijection public
open import VFP.Equality.Properties public
\end{code}

Injective functions.
%
\begin{code}
↪-reflexive : {A : Set} -> A ↪ A
↪-reflexive =
  record
  {  to  = λ x -> x
  ;  fro = λ x -> x
  ;  fro∘to≡id = λ x -> reflexive
  }

↪-transitive : {A B C : Set} -> A ↪ B -> B ↪ C -> A ↪ C
↪-transitive AB BC =
  record
  { to  = λ x -> to BC (to AB x)
  ; fro = λ x -> fro AB (fro BC x)
  ; fro∘to≡id = λ x -> transitive (congruent (fro AB) (fro∘to≡id BC (to  AB x))) (fro∘to≡id AB x)
  }
\end{code}

Bijections.
%
\begin{code}
≅-reflexive : {A : Set} -> A ≅ A
≅-reflexive =
  record
  {  to  = λ x -> x
  ;  fro = λ x -> x
  ;  fro∘to≡id = λ x -> reflexive
  ;  to∘fro≡id = λ x -> reflexive
  }

≅-symmetric : {A B : Set} -> A ≅ B -> B ≅ A
≅-symmetric AB =
  record
  {  to  = fro AB
  ;  fro = to  AB
  ;  fro∘to≡id = to∘fro≡id AB
  ;  to∘fro≡id = fro∘to≡id AB
  }

≅-transitive : {A B C : Set} -> A ≅ B -> B ≅ C -> A ≅ C
≅-transitive AB BC =
  record
  { to  = λ x -> to BC (to AB x)
  ; fro = λ x -> fro AB (fro BC x)
  ; fro∘to≡id = λ x -> transitive (congruent (fro AB) (fro∘to≡id BC (to  AB x))) (fro∘to≡id AB x)
  ; to∘fro≡id = λ x -> transitive (congruent (to  BC) (to∘fro≡id AB (fro BC x))) (to∘fro≡id BC x)
  }
\end{code}

Some random properties.
%
\begin{code}
open import VFP.Logic public

×-commutative : {A B : Set} → (A × B) ≅ (B × A)
×-commutative =
  record
  { to  = λ (a , b) → (b , a)
  ; fro = λ (a , b) → (b , a)
  ; fro∘to≡id = λ where (a , b) → reflexive
  ; to∘fro≡id = λ where (a , b) → reflexive
  }

_≅×≅_ : {A A' B B' : Set} -> (A ≅ A') -> (B ≅ B') -> ((A × B) ≅ (A' × B'))
iso-A ≅×≅ iso-B =
  record
  { to  = λ (a , b) → (to iso-A a , to iso-B b)
  ; fro = λ (a , b) → (fro iso-A a , fro iso-B b)
  ; fro∘to≡id = iso₁
  ; to∘fro≡id = iso₂
  } where
      iso₁ : (x : _ × _) → (fro iso-A (to iso-A (outl x)) , fro iso-B (to iso-B (outr x))) ≡ x
      iso₁ x rewrite fro∘to≡id iso-A (outl x)
             rewrite fro∘to≡id iso-B (outr x) = reflexive
      iso₂ : (x : _ × _) → (to iso-A (fro iso-A (outl x)) , to iso-B (fro iso-B (outr x))) ≡ x
      iso₂ x rewrite to∘fro≡id iso-A (outl x)
             rewrite to∘fro≡id iso-B (outr x) = reflexive
\end{code}

Laws of exponents.
%
\begin{code}
laws-of-exponents₁ : Extensionality -> ∀ (X : Set) -> (⊥ → X) ≅ ⊤
laws-of-exponents₁ extensional X = 
  record
  { to  = λ x → trivial
  ; fro = λ _ ()
  ; fro∘to≡id = λ where f → extensional λ ()
  ; to∘fro≡id = λ where trivial → reflexive
  }

laws-of-exponents₂ : Extensionality -> ∀ (X : Set) -> (⊤ → X) ≅ X
laws-of-exponents₂ extensional X = 
  record
  { to  = λ f → f trivial
  ; fro = λ where x trivial → x
  ; fro∘to≡id = λ f → extensional (λ where
                             trivial → reflexive)
  ; to∘fro≡id = λ where x → reflexive
  }

laws-of-exponents₃ : Extensionality -> ∀ (A B X : Set) -> (A ⊎ B → X) ≅ (A → X) × (B → X)
laws-of-exponents₃ extensional A B X = 
  record
  { to  = λ h → (λ a → h (inl a)) , λ b → h (inr b)
  ; fro = λ where
            (f , g) (inl x) → f x
            (f , g) (inr y) → g y
  ; fro∘to≡id = λ h → extensional (λ where
                              (inl x) → reflexive
                              (inr y) → reflexive)
  ; to∘fro≡id = λ where
                  (f , g) → reflexive
  }
\end{code}

Currying with dependent types (see definition of |to| and |fro| below).
%
\begin{code}
--open import Data.Product renaming (_,_ to infixr 6 _,_)
 
laws-of-exponents₄ : Extensionality -> ∀ (A : Set) -> ∀ (B : A -> Set) -> ∀ (X : Set) -> ((Σ A B) → X) ≅ ((a : A) → B a → X)
laws-of-exponents₄ extensional A B X =
  record
  { to  = λ h → λ a b → h (a , b)
  ; fro = λ k → λ where (a , b) → k a b
  ; fro∘to≡id = λ h → extensional λ x → reflexive
  ; to∘fro≡id = λ x → extensional (λ a → extensional (λ b → reflexive))
  }
\end{code}

  record
  { to = ?
  ; fro = ?
  ; fro∘to≡id = ?
  ; to∘fro≡id = ?
  }
