Bijectional reasoning in Wim Feijen's proof format.

\begin{code}
{-# OPTIONS --without-K --safe #-}

module VFP.Bijection.Reasoning where

open import VFP.Bijection.Properties public

infix  0 proof_
infixr 1 ≅-link
infix  2 _∎

proof_ : {A B : Set} → A ≅ B → A ≅ B
proof A≅B = A≅B

syntax ≅-link x q p = x ≅⟨ p ⟩ q
≅-link : (A : Set) -> {B C : Set} → B ≅ C → A ≅ B → A ≅ C
A ≅⟨ A≅B ⟩ B≅C = ≅-transitive A≅B B≅C

_∎ :  (A : Set) → A ≅ A
x ∎ = ≅-reflexive

by-definition : {A : Set} -> A ≅ A
by-definition = ≅-reflexive
\end{code}

    proof
      type1
    ≅⟨ why? ⟩ 
      type2
    ≅⟨ why? ⟩
      type3
    ∎

\begin{code}
module Examples where
  test : ∀ (A B : Set) → A × B ≅ B × A
  test A B =
    proof
      A × B
    ≅⟨ ×-commutative  ⟩ 
      B × A
    ∎

  open import VFP.Natural

--  data Less : ℕ → Set where
--    zero : {n : ℕ} →          Less (succ n)
--    succ : {n : ℕ} → Less n → Less (succ n)

  Less : ℕ → Set
  Less zero     = ⊥
  Less (succ n) = ⊤ ⊎ Less n

--  data Vector (V : Set) : ℕ → Set where
--    nil  :                              Vector V zero
--    cons : {n : ℕ} → V -> Vector V n -> Vector V (succ n)

  Vector : (V : Set) -> ℕ → Set
  Vector V zero     = ⊤
  Vector V (succ n) = V × Vector V n

  iso : Extensionality -> ∀ (V : Set) -> ∀ n -> (Less n -> V) ≅ Vector V n
  iso extensional V zero =
    proof
      (Less zero -> V)
    ≅⟨ ≅-reflexive  ⟩ 
      (⊥ -> V)
    ≅⟨ laws-of-exponents₁ extensional V ⟩ 
      ⊤
    ≅⟨ ≅-reflexive ⟩ 
      Vector V zero
    ∎
  iso extensional V (succ n) =
    proof
      (Less (succ n) -> V)
    ≅⟨ ≅-reflexive  ⟩ 
      (⊤ ⊎ Less n -> V)
    ≅⟨ laws-of-exponents₃ extensional ⊤ (Less n) V ⟩ 
      (⊤ -> V) × (Less n -> V)
    ≅⟨ laws-of-exponents₂ extensional V ≅×≅ iso extensional V n ⟩ 
      V × Vector V n
    ≅⟨ ≅-reflexive  ⟩ 
      Vector V (succ n)
    ∎
\end{code}
