Decide a property.

-------------------------------------------------------------------------------

\begin{code}
{-# OPTIONS --without-K --safe #-}

module VFP.Decidable where

open import VFP.Logic public

private variable
  A : Set
\end{code}

-------------------------------------------------------------------------------

\begin{code}
data Decide (P : Set) : Set where
  yes :   P -> Decide P
  no  : ¬ P -> Decide P

private variable
  P Q : Set
\end{code}

\begin{code}
map-Decide : P ↔ Q -> Decide P ↔ Decide Q
map-Decide (to & fro) =
    (λ { (yes p) → yes (to  p) ; (no ¬p)  → no (λ q → ¬p (fro q)) })
  & (λ { (yes q) → yes (fro q) ; (no ¬q)  → no (λ p → ¬q (to  p)) })
\end{code}

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

\begin{code}
tertium-non-datur : Decide P -> P ⊎ ¬ P
tertium-non-datur (yes p) = inl p
tertium-non-datur (no ¬p) = inr ¬p
\end{code}

\begin{code}
infix 0 if_then_else_
if_then_else_ : {P A : Set} -> Decide P → ({P} -> A) → ({¬ P} -> A) → A
if yes p then t else e = t { p}
if no ¬p then t else e = e {¬p}
\end{code}

-------------------------------------------------------------------------------

Logical connectives.

\begin{code}
infixr  8 _∨_
infixr  9 _∧_
infixr 10 ~_

_∨_ : Decide P -> Decide Q -> Decide (P ⊎ Q)
yes p ∨ yes q = yes (inl p)
yes p ∨ no ¬q = yes (inl p)
no ¬p ∨ yes q = yes (inr q)
no ¬p ∨ no ¬q = no (λ { (inl p) → ¬p p ; (inr q) → ¬q q })

_∧_ : Decide P -> Decide Q -> Decide (P × Q)
yes p ∧ yes q = yes (p , q)
yes p ∧ no ¬q = no (λ { (p , q) → ¬q q })
no ¬p ∧ yes q = no (λ { (p , q) → ¬p p })
no ¬p ∧ no ¬q = no (λ { (p , q) → ¬q q })

~_ : Decide P -> Decide (¬ P)
~ yes p = no (λ ¬p → ¬p p)
~ no ¬p = yes ¬p
\end{code}

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

A predicate P is decidable if P a is decidable for all a.

\begin{code}
Decidable : (A -> Set) -> Set
Decidable P = ∀ a -> Decide (P a)
\end{code}

-------------------------------------------------------------------------------
