Extrinsically ordered lists.

\begin{code}
--{-# OPTIONS --allow-unsolved-metas #-}
{-# OPTIONS --without-K --safe #-}

module VFP.List.Ordered
  {A : Set}
  (_≼_ : A → A → Set)
  (≼-transitive : ∀ {k m n} → k ≼ m → m ≼ n → k ≼ n)
  where

open import VFP.List --public DON'T re-export List
\end{code}

open import VFP.Order

module VFP.List.Ordered (𝕆 : Total-Order) where
open Total-Order 𝕆
private
  A = Carrier


-- private variable

-------------------------------------------------------------------------------

\begin{code}
_≼*_ : A -> List A -> Set
a ≼* xs = all (λ x -> a ≼ x) xs
\end{code}

\begin{code}
≼*-transitive : ∀ {x y ys} -> x ≼ y -> y ≼* ys -> x ≼* ys
≼*-transitive x≼y []            = []
≼*-transitive x≼y (y≼z ∷ y≼*zs) = ≼-transitive x≼y y≼z ∷ ≼*-transitive x≼y y≼*zs
\end{code}

\begin{code}
_*≼_ : List A → A → Set
as *≼ b = all (λ a → a ≼ b) as

*≼-transitive : ∀ {x y xs} -> xs *≼ x -> x ≼ y -> xs *≼ y
*≼-transitive []       q = []
*≼-transitive (p ∷ ps) q = ≼-transitive p q ∷ *≼-transitive ps q
\end{code}

\begin{code}
_*≼*_ : List A → List A → Set
as *≼* bs = all (λ a → a ≼* bs) as
\end{code}

- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

\begin{code}
data Ordered : List A -> Set where
  []  :                                      Ordered []
  _∷_ : ∀ {a as} -> a ≼* as -> Ordered as -> Ordered (a ∷ as)
\end{code}

\begin{code}
append-*≼ : ∀ {up} → ∀ {as : List A} → as *≼ up → ∀ {bs : List A} → bs *≼ up → (as ⧺ bs) *≼ up
append-*≼ []       qs = qs
append-*≼ (x ∷ ps) qs = x ∷ append-*≼ ps qs

≼*-append : ∀ {lo} → ∀ {as : List A} → lo ≼* as → ∀ {bs : List A} → lo ≼* bs → lo ≼* (as ⧺ bs)
≼*-append []       qs = qs
≼*-append (x ∷ ps) qs = x ∷ ≼*-append ps qs

⧺-preserves-ordered : {as : List A} → Ordered as → {bs : List A} → Ordered bs → as *≼* bs → Ordered (as ⧺ bs)
⧺-preserves-ordered []       bs []       = bs
⧺-preserves-ordered (a ∷ as) bs (p ∷ ps) = ≼*-append a p ∷ ⧺-preserves-ordered as bs ps
\end{code}

-------------------------------------------------------------------------------
