Basic type constructors aka logical connectives.

\begin{code}
{-# OPTIONS --without-K --safe #-}

module VFP.Logic where
\end{code}

-------------------------------------------------------------------------------

The empty type: absurdity/falsity.

\begin{code}
data ⊥ : Set where

ex-falso-quodlibet : ∀ {A : Set} -> ⊥ -> A
ex-falso-quodlibet ()
\end{code}

-------------------------------------------------------------------------------

The unit type: truth.

\begin{code}
record ⊤ : Set where
  instance constructor trivial
\end{code}

-------------------------------------------------------------------------------

Sums (aka coproducts aka disjoint unions): disjunction.

\begin{code}
infixr 3 _⊎_

data _⊎_ (A : Set) (B : Set) : Set where
  inl : (x : A) → A ⊎ B
  inr : (y : B) → A ⊎ B
\end{code}

-------------------------------------------------------------------------------

Reintroduce non-dependent pairs? Hmm, this poses problems with let-bindings:
cannot split on argument of unresolved type, see Comparator.

Products (aka pairs): conjunction.
%
begin{code}
infixr 4 _×_
infixr 6 _,_

record _×_ (A : Set) (B : Set) : Set where
  constructor _,_
  field outl : A
        outr : B
open _×_ public
end{code}

Products (aka pairs): conjunction, defined in terms of dependent
products (aka dependent sums).

\begin{code}
infixr 4 _×_
infixr 6 _,_

record Σ (A : Set) (B : A → Set) : Set where
  constructor _,_
  field outl : A
        outr : B outl
open Σ public

_×_ : ∀ (A : Set) (B : Set) → Set
A × B = Σ A (λ x -> B)

∃ : ∀ {A : Set} → (A → Set) → Set
∃ = Σ _
\end{code}

Pairs for the next universe level.

\begin{code}
module Next-Universe where
  infixr 4 _×₁_

  record Σ₁ (A : Set₁) (B : A → Set₁) : Set₁ where
    constructor _,_
    field outl : A
          outr : B outl
  open Σ₁ public

  _×₁_ : ∀ (A : Set₁) (B : Set₁) → Set₁
  A ×₁ B = Σ₁ A (λ x -> B)
\end{code}

-------------------------------------------------------------------------------

Negation.

\begin{code}
infix 5 ¬_

¬_ : Set → Set
¬ P = P → ⊥
\end{code}

-------------------------------------------------------------------------------

Implication, reverse implication, and bi-implication aka logical
equivalence.

OR: infixr 2 _⟹_ _⟸_ _⟺_

NB. The precedence of |->| is lower than any precedence you may
declare with |infixl| and |infixr|.

\begin{code}
infixr 2 _←_ _↔_
infixr 6 _&_

_←_ : Set -> Set -> Set
A ← B = B → A

record _↔_ (A B : Set) : Set where
  constructor _&_
  field to  : A → B
        fro : A ← B
open _↔_ public
\end{code}

OR: _iff_
 
-------------------------------------------------------------------------------

Decide a property.

begin{code}
data Decide (P : Set) : Set where
  yes :   P -> Decide P
  no  : ¬ P -> Decide P

infix 0 if_then_else_
if_then_else_ : {P A : Set} -> Decide P → ({P} -> A) → ({¬ P} -> A) → A
if yes p then t else e = t { p}
if no ¬p then t else e = e {¬p}
end{code}

-------------------------------------------------------------------------------

Unicode table:

⊥   \bot            U-22a5  UP TACK
⊤   \top            U-22a4  DOWN TACK
⊎   \u+             U-228e  MULTISET UNION
×   \x              U-00d7  MULTIPLICATION SIGN
¬   \neg            U-00ac  NOT SIGN
⟹  \r              U-27f9 LONG RIGHTWARDS DOUBLE ARROW
⟸  \l              U-27f8 LONG LEFTWARDS DOUBLE ARROW
⟺  \lr             U-27fa LONG LEFT RIGHT DOUBLE ARROW
∃   \ex
