Properties of logical connectives. A fairly random collection of
(non-) properties.

-------------------------------------------------------------------------------

\begin{code}
{-# OPTIONS --without-K --safe #-}

module VFP.Logic.Properties where

open import VFP.Logic public

private variable
  A B C : Set
\end{code}

-------------------------------------------------------------------------------

Implication.

\begin{code}
→-reflexive : A → A
→-reflexive = λ x -> x

→-transitive : (A → B) -> (B → C) -> (A → C)
→-transitive AB BC = λ x -> BC (AB x)
\end{code}

Equivalence.

\begin{code}
↔-reflexive : A ↔ A
↔-reflexive = (λ x -> x) & (λ x -> x)

↔-symmetric : A ↔ B -> B ↔ A
↔-symmetric AB = fro AB & to AB

↔-transitive : A ↔ B -> B ↔ C -> A ↔ C
↔-transitive AB BC = (λ x -> to BC (to AB x)) & (λ x -> fro AB (fro BC x))
\end{code}

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

\begin{code}
×-commutative : (A × B) ↔ (B × A)
×-commutative = (λ (a , b) → (b , a)) & (λ (b , a) → (a , b))
\end{code}

<<distributivity>>
\begin{code}
×-left-distributes-over-⊎ : (A ⊎ B) × C ↔ A × C ⊎ B × C
×-left-distributes-over-⊎ = (λ where
                               (inl a , c) → inl (a , c)
                               (inr b , c) → inr (b , c))
                          & (λ where
                               (inl (a , c)) → inl a , c
                               (inr (b , c)) → inr b , c)
\end{code}

-------------------------------------------------------------------------------

Interdefinability.

<<interdefinability>>
\begin{code}
conjunction-as-disjunction : A × B -> ¬ (¬ A ⊎ ¬ B)
conjunction-as-disjunction (a , b) (inl ¬a) = ¬a a
conjunction-as-disjunction (a , b) (inr ¬b) = ¬b b

-- conjunction-as-disjunctionᴿ : A × B ← ¬ (¬ A ⊎ ¬ B)  -- good luck!

disjunction-as-conjunction : A ⊎ B -> ¬ (¬ A × ¬ B)
disjunction-as-conjunction (inl a) (¬a , ¬b) = ¬a a
disjunction-as-conjunction (inr b) (¬a , ¬b) = ¬b b

-- disjunction-as-conjunctionᴿ : A ⊎ B ← ¬ (¬ A × ¬ B)  -- good luck!

-- implication-as-disjunction : (A → B) → ¬ A ⊎ B  -- good luck!
\end{code}

-------------------------------------------------------------------------------

<<contraposition>>
\begin{code}
contrapositive : (A → B) → (¬ B → ¬ A)
contrapositive = λ f ¬b a → ¬b (f a)  -- composition

-- contrapositiveᴿ : (A → B) ← (¬ B → ¬ A)  -- good luck!

flip : (A -> ¬ B) -> (B -> ¬ A)
flip f b a = f a b
\end{code}

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

The classical law A ⊎ ¬ A, tertium non datur, (latin for „ein Drittes
gibt es nicht“, law of excluded middle), does *not* hold
intuitionistically. It is, however, irrefutable.

<<excluded-middle-irrefutable>>
\begin{code}
-- excluded-middle : A ⊎ ¬ A  -- good luck!

excluded-middle-irrefutable : ¬ ¬ (A ⊎ ¬ A)
excluded-middle-irrefutable =
  λ neg-excluded-middle → neg-excluded-middle (inr (λ a → neg-excluded-middle (inl a)))
\end{code}

<<excluded-middle-irrefutable-var>>
\begin{code}
excluded-middle-irrefutable′ : ¬ ¬ (A ⊎ ¬ A)
excluded-middle-irrefutable′ neg-excluded-middle =
  neg-excluded-middle (inr (λ a → neg-excluded-middle (inl a)))
\end{code}

\begin{code}
simpler-version : ¬ (A ⊎ ¬ A) -> A ⊎ ¬ A
simpler-version = λ neg-excluded-middle → inr (λ a → neg-excluded-middle (inl a))
\end{code}

We show at the end:

double-negation-irrefutable : ¬ ¬ (¬ ¬ A -> A)

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

De Morgan's laws and co. In classicial logic they can be used to turn
a formula into negation normalform.

<<single-negation>>
\begin{code}
negation-⊥ : ¬ ⊥ ↔ ⊤
negation-⊥ = (λ p → trivial) & (λ p x → x)

negation-⊤ : ¬ ⊤ ↔ ⊥
negation-⊤ = (λ z → z trivial) & (λ z x → z) 

-- negation-¬ : ¬ ¬ A -> A  -- good luck!
negation-¬ᴿ : ¬ ¬ A ← A
negation-¬ᴿ a ¬a = ¬a a  -- continuations at work

negation-⊎ : ¬ (A ⊎ B) ↔ ¬ A × ¬ B
negation-⊎ = (λ f → (λ a → f (inl a)) , (λ b → f (inr b)))
      & (λ where (¬a , ¬b) (inl a) → ¬a a
                 (¬a , ¬b) (inr b) → ¬b b)

-- negation-× : ¬ (A × B) -> ¬ A ⊎ ¬ B  -- good luck!
negation-×ᴿ : ¬ (A × B) ← ¬ A ⊎ ¬ B
negation-×ᴿ (inl a) = λ p → a (outl p)
negation-×ᴿ (inr b) = λ p → b (outr p)
\end{code}

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

The classical law ¬ ¬ A -> A, duplex negatio affirmat (latin „die
doppelte Verneinung bekräftigt/bejaht“, logical double negation),
does *not* hold intuitionistically.

<<double-negation>>
\begin{code}
-- double-negation : ¬ ¬ A -> A  -- good luck!

double-negationᴿ : ¬ ¬ A ← A  -- same as negation-¬ᴿ
double-negationᴿ a ¬a = ¬a a

triple-negation : ¬ ¬ ¬ A -> ¬ A
triple-negation = λ ¬¬¬a a → ¬¬¬a (λ ¬a → ¬a a)

triple-negationᴿ : ¬ ¬ ¬ A ← ¬ A
triple-negationᴿ = double-negationᴿ
\end{code}

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

Gentzen's double negation translation, see also
VFP.Logic.Propositional.Natural-Deduction.Double-Negation.

<<double-negation-translation>>
\begin{code}
double-negation-⊥ : ¬ ¬ ⊥ -> ⊥
double-negation-⊥ = λ c → c (λ x → x)

double-negation-⊥ᴿ : ¬ ¬ ⊥ ← ⊥
double-negation-⊥ᴿ = λ z z₁ → z₁ z

double-negation-⊤ : ¬ ¬ ⊤ -> ⊤
double-negation-⊤ = λ x → trivial

double-negation-⊤ᴿ : ¬ ¬ ⊤ ← ⊤
double-negation-⊤ᴿ = λ _ z → z trivial

-- double-negation-⊎ : ¬ ¬ (A ⊎ B) -> ¬ ¬ A ⊎ ¬ ¬ B  -- good luck!
double-negation-⊎ : ¬ ¬ (A ⊎ B) -> ¬ (¬ ¬ ¬ A × ¬ ¬ ¬ B)
double-negation-⊎ f (¬¬¬a , ¬¬¬b) = f (λ where
                                         (inl a) → ¬¬¬a (λ ¬a → ¬a a)
                                         (inr b) → ¬¬¬b (λ ¬b → ¬b b))
double-negation-⊎ᴿ : ¬ ¬ (A ⊎ B) ← ¬ (¬ ¬ ¬ A × ¬ ¬ ¬ B)
double-negation-⊎ᴿ = λ z z₁ → z ((λ x → x (λ x₁ → z₁ (inl x₁))) , (λ x → x (λ x₁ → z₁ (inr x₁))))

double-negation-× : ¬ ¬ (A × B) -> (¬ ¬ A × ¬ ¬ B)
double-negation-× f = (λ ¬a → f (λ (a , b) → ¬a a)) , (λ ¬b → f (λ (a , b) → ¬b b))

double-negation-×ᴿ : ¬ ¬ (A × B) ← (¬ ¬ A × ¬ ¬ B)
double-negation-×ᴿ = λ z z₁ → outl z (λ x → outr z (λ x₁ → z₁ (x , x₁)))

double-negation-→ : ¬ ¬ (A → B) -> (¬ ¬ A → ¬ ¬ B)
double-negation-→ ¬¬f = λ ¬¬a ¬b → ¬¬a (λ a → ¬¬f (λ f → ¬b (f a)))

double-negation-→ᴿ : ¬ ¬ (A → B) ← (¬ ¬ A → ¬ ¬ B)
double-negation-→ᴿ f = λ x → x (λ a → ex-falso-quodlibet (f (λ c → c a) (λ b → x (λ _ → b))))
\end{code}

NB. A formula is called *stable* if double negation elimination holds for it.

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

TODO: Examples of double negation translation.

NB. excluded-middle-irrefutable is actually *not* the double negation of
excluded middle, as A is universally quantified: ¬ ¬ (∀ {A : Set} → A ⊎ ¬ A).

¬₁_ : Set₁ → Set₁
¬₁ P = P → ⊥

double-negation-dn : ¬₁ ¬₁ (∀ {A : Set} → ¬ ¬ A -> A)
double-negation-dn = {!!}

double-negation-em : ¬₁ ¬₁ (∀ (A : Set) → A ⊎ ¬ A)
double-negation-em = λ negation-excluded-middle → negation-excluded-middle (λ A -> inr (λ (a : A) → negation-excluded-middle ({!a!})))

-------------------------------------------------------------------------------

\begin{code}
excluded-middle→double-negation-elimination : (A ⊎ ¬ A) -> (¬ ¬ A → A)
excluded-middle→double-negation-elimination (inl  a) c = a
excluded-middle→double-negation-elimination (inr ¬a) c = ex-falso-quodlibet (c ¬a)

¬¬-monotone : (A -> B) -> (¬ ¬ A -> ¬ ¬ B)
¬¬-monotone f = λ ¬¬a ¬b → ¬¬a (λ a → ¬b (f a))

double-negation-irrefutable : ¬ ¬ (¬ ¬ A -> A)
double-negation-irrefutable = ¬¬-monotone excluded-middle→double-negation-elimination excluded-middle-irrefutable
\end{code}

-------------------------------------------------------------------------------

Further reading:

●  https://agda.readthedocs.io
●  https://ncatlab.org/nlab/show/double+negation+translation
●  https://math.stackexchange.com/questions/658262/what-is-the-correct-reading-of-bot??
