\begin{code}
module VFP.Logic.Reasoning where

open import VFP.Logic.Properties public
\end{code}

begin{code}
infix 0 proof_
proof_ : {A B : Set} → (A → B) → (A → B)
proof f = f

infixr 1 →-link ↔-link
syntax →-link x q p = x →⟨ p ⟩ q
syntax ↔-link x q p = x ↔⟨ p ⟩ q
→-link : (A : Set) {B C : Set} → (B → C) → (A → B) → (A → C)
→-link _ g f = λ a → g (f a)   -- composition
↔-link : (A : Set) {B C : Set} → (B → C) → (A ↔ B) → (A → C)
↔-link _ g f = λ a → g (to f a)

infix 2 _∎
_∎ : (A : Set) → (A → A)
x ∎ = λ a → a   -- identity
end{code}

\begin{code}
private
  data Order : Set -> Set -> Set where
    LE : {A B : Set} -> (A → B) -> Order A B
    EQ : {A B : Set} -> (A ↔ B) -> Order A B

module _ {A B : Set} where
  ⟦_⟧ : Order A B -> Set
  ⟦ LE  _ ⟧ = A → B
  ⟦ EQ  _ ⟧ = A ↔ B

  infix 0 proof_

  proof_ : (p : Order A B) → ⟦ p ⟧
  proof (LE p) = p
  proof (EQ p) = p

infixr 1 →-link ↔-link
infix  2 _∎
syntax →-link x q p = x →⟨ p ⟩ q
syntax ↔-link x q p = x ↔⟨ p ⟩ q


infix 0 case_of_
case_of_ : {A : Set} {B : Set} → A → (A → B) → B
case x of f = f x

→-link : (A : Set) {B C : Set} → Order B C → (A → B) → Order A C
→-link x (LE g) f = LE (→-transitive f g)
→-link x (EQ g) f = LE (λ x -> to g (f x))

↔-link : (A : Set) {B C : Set} → Order B C → (A ↔ B) → Order A C
↔-link x (LE g) f = LE (λ x → g (to f x))
↔-link x (EQ g) f = EQ (↔-transitive f g)

_∎ : (A : Set) → Order A A
x ∎ = EQ ↔-reflexive
\end{code}
