\begin{code}
{-# OPTIONS --without-K --safe #-}

module VFP.Natural.Properties where

open import VFP.Natural public
open import VFP.Equality.Properties public

private variable
  k m n : ℕ
\end{code}

\begin{code}
succ-inj : ∀ {m n : ℕ} -> succ m ≡ succ n -> m ≡ n
succ-inj reflexive = reflexive
\end{code}

-------------------------------------------------------------------------------

Additive monoid.

<<additive-monoid>>
\begin{code}
+-left-identity : ∀ {n} -> zero + n ≡ n
+-left-identity = reflexive

+-right-identity : ∀ {n} -> n + zero ≡ n
+-right-identity {zero}    = reflexive
+-right-identity {succ n}  = congruent succ +-right-identity

+-associative : ∀ k {m n} -> (k + m) + n ≡ k + (m + n)
+-associative (zero)   = reflexive
+-associative (succ k) = congruent succ (+-associative k)
\end{code}

NB. The first argument is explicit to avoid unresolved metas.

\begin{code}
shift : ∀ m n -> m + succ n ≡ succ (m + n)
shift zero n = reflexive
shift (succ m) n = congruent succ (shift m n)

shift' : ∀ m n -> succ (m + n) ≡ m + succ n
shift' zero n = reflexive
shift' (succ m) n = congruent succ (shift' m n)

+-commutative : ∀ m n -> m + n ≡ n + m
+-commutative zero n = symmetric +-right-identity
+-commutative (succ m) n = transitive (congruent succ (+-commutative m n)) (shift' n m)
\end{code}

<<additive-monoid-reverse>>
\begin{code}
+ᴿ-left-identity : ∀ n -> zero +ᴿ n ≡ n
+ᴿ-left-identity (zero)   = reflexive
+ᴿ-left-identity (succ n) = congruent succ (+ᴿ-left-identity n)

+ᴿ-right-identity : ∀ n -> n +ᴿ zero ≡ n
+ᴿ-right-identity n = reflexive

+ᴿ-associative : ∀ k m n -> (k +ᴿ m) +ᴿ n ≡ k +ᴿ (m +ᴿ n)
+ᴿ-associative k m (zero)   = reflexive
+ᴿ-associative k m (succ n) = congruent succ (+ᴿ-associative k m n)
\end{code}

<<exercise-plus>>
\begin{code}
shiftᴿ : ∀ m n -> succ (m +ᴿ n) ≡ succ m +ᴿ n
shiftᴿ m (zero)   = reflexive
shiftᴿ m (succ n) = congruent succ (shiftᴿ m n)

+≡+ᴿ : ∀ m n -> m + n ≡ m +ᴿ n
+≡+ᴿ (zero)   n = symmetric (+ᴿ-left-identity n)
+≡+ᴿ (succ m) n = transitive (congruent succ (+≡+ᴿ m n)) (shiftᴿ m n)
\end{code}

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

Multiplicative monoid.

\begin{code}
*-left-identity : ∀ n -> succ zero * n ≡ n
*-left-identity n = +-right-identity

*-right-identity : ∀ n -> n * succ zero ≡ n
*-right-identity zero = reflexive
*-right-identity (succ n) = congruent succ (*-right-identity n)
\end{code}

\begin{code}
*-left-zero : ∀ n -> zero * n ≡ zero
*-left-zero n = reflexive

*-left-distributes-over-+ : ∀ n a b -> (a + b) * n ≡ a * n + b * n
*-left-distributes-over-+ n zero b = reflexive
*-left-distributes-over-+ n (succ a) b = transitive (congruent (λ hole → n + hole) (*-left-distributes-over-+ n a b)) (symmetric (+-associative n)) -- (a * n) (b * n)))

middle-two-interchange : ∀ a b c d -> (a + b) + (c + d) ≡ (a + c) + (b + d)
--middle-two-interchange a b c d = transitive (+-associative a b (c + d)) (transitive (congruent (λ x → a + x) (transitive (symmetric (+-associative b c d)) (transitive (congruent (λ x → x + d) (+-commutative b c)) (+-associative c b d)))) (symmetric (+-associative a c (b + d))))
middle-two-interchange a b c d = transitive (+-associative a) (transitive (congruent (λ x → a + x) (transitive (symmetric (+-associative b)) (transitive (congruent (λ x → x + d) (+-commutative b c)) (+-associative c)))) (symmetric (+-associative a))) -- a c (b + d))))

*-right-zero : ∀ n -> n * zero ≡ zero
*-right-zero (zero)   = reflexive
*-right-zero (succ n) = *-right-zero n

*-right-distributes-over-+ : ∀ n a b -> n * (a + b) ≡ n * a + n * b
*-right-distributes-over-+ (zero)   a b = reflexive
*-right-distributes-over-+ (succ n) a b = transitive (congruent (λ x → a + b + x) (*-right-distributes-over-+ n a b))
                                               (middle-two-interchange a b (n * a) (n * b))
*-associative : ∀ k m n -> (k * m) * n ≡ k * (m * n)
*-associative zero m n = reflexive
*-associative (succ k) m n =
  transitive (*-left-distributes-over-+ n m (k * m)) (congruent (λ hole → m * n + hole) (*-associative k m n))
\end{code}

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

Properties of integer/natural division by 2.

Inverses.

\begin{code}
⌊/2⌋∘2*≡id : ∀ n → ⌊ (2* n) /2⌋ ≡ n
⌊/2⌋∘2*≡id (zero)   = reflexive
⌊/2⌋∘2*≡id (succ n) = congruent succ (⌊/2⌋∘2*≡id n)
\end{code}

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

Maximum.

\begin{code}
⊔-idempotent : ∀ {n} → n ⊔ n ≡ n
⊔-idempotent {zero}   = reflexive
⊔-idempotent {succ n} = congruent succ ⊔-idempotent

⊔-idempotent+1 : ∀ {n} → succ n ⊔ n ≡ succ n
⊔-idempotent+1 {zero}   = reflexive
⊔-idempotent+1 {succ n} = congruent succ ⊔-idempotent+1

⊔-1+idempotent : ∀ {n} → n ⊔ succ n ≡ succ n
⊔-1+idempotent {zero}   = reflexive
⊔-1+idempotent {succ n} = congruent succ ⊔-1+idempotent
\end{code}

-------------------------------------------------------------------------------
