Some example applications of codecs.

-------------------------------------------------------------------------------

\begin{code}
module VFP.Serialise.Indexed.Example where

open import VFP.Serialise.Indexed.Codec
open import VFP.Serialise.Indexed.Huffman
open import VFP.Character
open import VFP.Equality
open import VFP.Logic
open import VFP.Maybe using (Maybe ; Nothing ; Just)
import VFP.Maybe as Maybe
open import VFP.List using (List ; [] ; _∷_ ; length; mapMaybe)
open import VFP.Vector
open import VFP.Natural.Non-Strict-Order
\end{code}

-------------------------------------------------------------------------------

\begin{code}
code : Code-Tree Character
\end{code}

<<codetree>>
\begin{code}
code =  ((⌊ 'h' ⌋ ⋏ ⌊ 't' ⌋) ⋏ ⌊ 'e' ⌋)
          ⋏ ((⌊ 'i' ⌋ ⋏ ⌊ 'n' ⌋) ⋏ ((⌊ 'v' ⌋ ⋏ ⌊ 'g' ⌋) ⋏ ⌊ blank ⌋))
\end{code}

<<i-in-code>>
\begin{code}
i-am-in  :  'i' ∈ code
i-am-in  =  right (left (left here))
\end{code}

\begin{code}
lookup : (c : Character) -> Maybe (c ∈ code)
lookup 'e' = Just (left (right here))
lookup 'g' = Just (right (right (left (right here))))
lookup 'h' = Just (left (left (left here)))
lookup 'i' = Just (right (left (left here)))
lookup 'n' = Just (right (left (right here)))
lookup 't' = Just (left (left (right here)))
lookup 'v' = Just (right (right (left (left here))))
lookup '␣' = Just (right (right (right here)))
lookup _   = Nothing

certify : Character -> Maybe (In code)
certify c = Maybe.map (λ p → c , p) (lookup c)
\end{code}

\begin{code}
fromList : {A : Set} (xs : List A) -> Vector A (length xs)
fromList []       = ⟨⟩
fromList (x ∷ xs) = x , fromList xs
\end{code}

\begin{code}
raw-text : List Character
raw-text = 'e' ∷ 'i' ∷ 'g' ∷ 'h' ∷ 't' ∷ '␣' ∷ 'i' ∷ 'n' ∷ '␣' ∷ 't' ∷ 'h' ∷ 'e' ∷ '␣' ∷ 'e' ∷ 'v' ∷ 'e' ∷ 'n' ∷ 'i' ∷ 'n' ∷ 'g' ∷ []

text : Vector (In code) 20
text = fromList (mapMaybe certify raw-text)
\end{code}

<<compression>>
\begin{code}
_  :  size (Codec.encode (𝕍𝕖𝕔𝕥𝕠𝕣 (𝕀𝕟 code) 20) text []) ≡ 59
_  =  reflexive

_  :  size (Codec.encode (𝕃𝕚𝕤𝕥′ (𝕀𝕟 code)) (20 , text) []) ≡ 67
_  =  reflexive
\end{code}

\begin{code}
compressed-text = Codec.encode (𝕍𝕖𝕔𝕥𝕠𝕣 (𝕀𝕟 code) 20) text []
_ : size compressed-text ≡ 59
_ = reflexive

compressed-text′ = Codec.encode (𝕃𝕚𝕤𝕥′ (𝕀𝕟 code)) (20 , text) []
_ : size compressed-text′ ≡ 67
_ = reflexive
\end{code}

Ctrl-C Ctrl-N
  bits compressed-text′
  size compressed-text′

\begin{code}
encoding : Bits 67
encoding = 𝟣 (𝟢 (𝟣 (𝟣 (𝟢 (𝟣 (𝟢 (𝟢 (𝟢 (𝟣 (𝟣 (𝟢 (𝟢 (𝟣 (𝟣 (𝟢 (𝟣 (𝟢 (𝟢 (𝟢 (
           𝟢 (𝟢 (𝟣 (𝟣 (𝟣 (𝟣 (𝟣 (𝟢 (𝟢 (𝟣 (𝟢 (𝟣 (𝟣 (𝟣 (𝟣 (𝟢 (𝟢 (𝟣 (𝟢 (𝟢 (
           𝟢 (𝟢 (𝟣 (𝟣 (𝟣 (𝟣 (𝟢 (𝟣 (𝟣 (𝟣 (𝟢 (𝟢 (𝟢 (𝟣 (𝟣 (𝟢 (𝟣 (𝟣 (𝟢 (𝟢 (
           𝟣 (𝟢 (𝟣 (𝟣 (𝟣 (𝟢 (𝟣 []))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
\end{code}

-------------------------------------------------------------------------------
