Ternary numbers as a sequence of trits (trinary digits).

\begin{code}
{-# OPTIONS --without-K --safe #-}

module VFP.Ternary where

\end{code}

-------------------------------------------------------------------------------

<<Ternary>>
\begin{code}
data Ternary : Set where
  𝟘t : Ternary
  _𝟙 _𝟚 _𝟛 : Ternary -> Ternary

Eau-de-Cologne = 𝟘t 𝟙 𝟚 𝟛 𝟛 𝟛 𝟙 𝟙 𝟙
\end{code}

We use postfix notation for the digits.

--  _𝟣 _𝟤 _𝟥 : Ternary -> Ternary -- Agda cannot parse 𝟢b 𝟣 𝟥 𝟤, why?

\begin{code}
succ : Ternary -> Ternary
succ (𝟘t)  = 𝟘t 𝟙
succ (n 𝟙) = n 𝟚
succ (n 𝟚) = n 𝟛
succ (n 𝟛) = succ n 𝟙
\end{code}

\begin{code}
_𝟘 : Ternary -> Ternary  -- aka shift
(𝟘t)  𝟘 = 𝟘t
(n 𝟙) 𝟘 = (n 𝟘) 𝟛
(n 𝟚) 𝟘 = (n 𝟙) 𝟛
(n 𝟛) 𝟘 = (n 𝟚) 𝟛
\end{code}

\begin{code}
open import VFP.Natural renaming (zero to z ; succ to s)

from-Natural : Natural -> Ternary
from-Natural (z)   = 𝟘t
from-Natural (s n) = succ (from-Natural n)

to-Natural : Ternary -> Natural
to-Natural (𝟘t)  = 0
to-Natural (n 𝟙) = 1 + to-Natural n * 3
to-Natural (n 𝟚) = 2 + to-Natural n * 3
to-Natural (n 𝟛) = 3 + to-Natural n * 3
\end{code}

TODO: improve proof.

\begin{code}
open import VFP.Equality.Properties

aux : ∀ n -> to-Natural (succ n) ≡ s (to-Natural n)
aux (𝟘t)  = reflexive
aux (n 𝟙) = reflexive
aux (n 𝟚) = reflexive
aux (n 𝟛) = congruent (λ hole → s (hole * 3)) (aux n)

iso : ∀ n -> to-Natural (from-Natural n) ≡ n
iso (z)   = reflexive
iso (s n) = transitive (aux (from-Natural n)) (congruent s (iso n))

yup : ∀ n -> succ n 𝟘 ≡ n 𝟛
yup 𝟘t = reflexive
yup (n 𝟙) = reflexive
yup (n 𝟚) = reflexive
yup (n 𝟛) rewrite yup n = reflexive

succ-𝟘≡𝟙 : ∀ n -> succ (n 𝟘) ≡ (n 𝟙)
succ-𝟘≡𝟙 𝟘t = reflexive
succ-𝟘≡𝟙 (n 𝟙) rewrite succ-𝟘≡𝟙 n = reflexive
succ-𝟘≡𝟙 (n 𝟚) rewrite succ-𝟘≡𝟙 n = reflexive
succ-𝟘≡𝟙 (n 𝟛) rewrite succ-𝟘≡𝟙 n = reflexive

xua : ∀ n -> from-Natural (n * 3) ≡ (from-Natural n) 𝟘
xua z = reflexive
xua (s n) = transitive (congruent (λ hole -> succ (succ (succ hole))) (xua n)) (transitive (congruent (λ hole -> succ (succ hole)) (succ-𝟘≡𝟙 (from-Natural n))) (symmetric (yup (from-Natural n))))

osi : ∀ n -> from-Natural (to-Natural n) ≡ n
osi (𝟘t)  = reflexive
osi (n 𝟙) = transitive (congruent succ (xua (to-Natural n))) (transitive (congruent (λ hole -> succ (hole 𝟘)) (osi n)) (succ-𝟘≡𝟙 n))
osi (n 𝟚) = transitive (congruent (λ hole -> succ (succ hole)) (xua (to-Natural n))) (transitive (congruent (λ hole -> succ (succ (hole 𝟘))) (osi n)) (congruent succ (succ-𝟘≡𝟙 n)))
osi (n 𝟛) = transitive (congruent (λ hole -> succ (succ (succ hole))) (xua (to-Natural n))) (transitive (congruent (λ hole -> succ (succ (succ (hole 𝟘)))) (osi n)) (congruent (λ hole → succ (succ hole)) (succ-𝟘≡𝟙 n)))

open import VFP.Bijection

Natural≅Ternary : Natural ≅ Ternary
Natural≅Ternary = record
  { to = from-Natural
  ; fro = to-Natural
  ; fro∘to≡id = iso
  ; to∘fro≡id = osi }

\end{code}

-------------------------------------------------------------------------------
