\begin{code}
{-# OPTIONS --without-K --safe #-}

module VFP.Vector where

open import VFP.Natural public

private variable
  m n : ℕ
  Elem : Set
\end{code}

Vectors as iterated products.

-------------------------------------------------------------------------------

\begin{code}
open import VFP.Logic hiding (_,_)

Vector′ : ℕ → Set → Set
Vector′ (zero)   Elem = ⊤
Vector′ (succ n) Elem = Elem × Vector′ n Elem
\end{code}

. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

\begin{code}
infixr 6 _,_
\end{code}

<<Vector>>
\begin{code}
data Vector (Elem : Set) : Natural → Set where
  ⟨⟩   : Vector Elem zero
  _,_  : Elem → Vector Elem n → Vector Elem (succ n)
\end{code}

\begin{code}
𝕍 = Vector
\end{code}

\begin{code}
map : ∀ {A B} -> (A -> B) -> (Vector A n -> Vector B n)
map f ⟨⟩       = ⟨⟩
map f (x , xs) = f x , map f xs
\end{code}

\begin{code}
_⧺_ : Vector Elem m -> Vector Elem n -> Vector Elem (m + n)
⟨⟩       ⧺ ys = ys
(x , xs) ⧺ ys = x , (xs ⧺ ys)
\end{code}

-------------------------------------------------------------------------------

Bounded vectors: index is an upper bound on the length of the vector.

\begin{code}
data Vector< (Elem : Set) : Natural -> Set where
  ⟨⟩   : Vector< Elem n   --- NB. |n| *not* |zero|
  _,_ : Elem -> Vector< Elem n -> Vector< Elem (succ n)
\end{code}

Padding: relaxing the upper bound.

\begin{code}
open import VFP.Natural.Properties

pad-r : Vector< Elem m -> Vector< Elem (m + n)
pad-r ⟨⟩       = ⟨⟩  -- the empty vectors have different implicit arguments
pad-r (x , xs) = x , pad-r xs

pad-l : Vector< Elem n -> Vector< Elem (m + n)
pad-l {n = n} {m = m} xs rewrite +-commutative m n = pad-r xs
\end{code}


-------------------------------------------------------------------------------
